@@ -984,6 +984,25 @@ main (int argc, char **argv)
     g_value_array_free (vals_ret);
   }
 
+  {
+    const gchar *in_sig = "a(iou)sq";
+    gchar *out_sig = NULL;
+
+    g_print ("Calling EchoSignature: %s\n", in_sig);
+    if (!org_freedesktop_DBus_GLib_Tests_MyObject_echo_signature (proxy,
+          in_sig, &out_sig, &error))
+      lose_gerror ("Failed to complete EchoSignature call", error);
+
+    if (out_sig == NULL)
+      lose ("EchoSignature returned NULL");
+    if (strcmp (in_sig, out_sig) != 0)
+      lose ("EchoSignature changed the signature");
+
+    g_print ("EchoSignature returned: %s\n", out_sig);
+
+    g_free (out_sig);
+  }
+
   {
     GValue *val;
     GHashTable *table;
@@ -1309,6 +1328,109 @@ main (int argc, char **argv)
     g_mem_profile ();
   }
 
+  for (i=0; i<3; i++)
+  {
+    GHashTable *table;
+    GHashTable *ret_table = NULL;
+    const gchar *foo[] = { "foo", NULL };
+    const gchar *bar[] = { "bar", "baz", NULL };
+    const gchar **ret_foo = NULL, **ret_bar = NULL;
+
+    table = g_hash_table_new (g_str_hash, g_str_equal);
+    g_hash_table_insert (table, "dub", foo);
+    g_hash_table_insert (table, "sox", bar);
+
+    g_print ("Calling DictOfSigs\n");
+
+    if (!org_freedesktop_DBus_GLib_Tests_MyObject_dict_of_sigs (proxy, table,
+          &ret_table, &error))
+      lose_gerror ("Failed to complete DictOfSigs call", error);
+
+    if (ret_table == NULL)
+      lose ("DictOfSigs didn't return a hash table");
+
+    if (g_hash_table_size (ret_table) != 2)
+      lose ("DictOfSigs has too many entries");
+
+    ret_foo = g_hash_table_lookup (ret_table, "dub");
+    ret_bar = g_hash_table_lookup (ret_table, "sox");
+
+    if (ret_foo == NULL || ret_bar == NULL)
+      lose ("DictOfSigs is missing entries");
+
+    if (ret_foo[0] == NULL ||
+        ret_foo[1] != NULL ||
+        strcmp (ret_foo[0], "foo") != 0)
+      lose ("DictOfSigs mangled foo");
+
+    if (ret_bar[0] == NULL ||
+        ret_bar[1] == NULL ||
+        ret_bar[2] != NULL ||
+        strcmp (ret_bar[0], "bar") != 0 ||
+        strcmp (ret_bar[1], "baz") != 0)
+      lose ("DictOfSigs mangled bar");
+
+    g_hash_table_destroy (table);
+    g_hash_table_destroy (ret_table);
+
+    g_mem_profile ();
+  }
+
+  for (i=0; i<3; i++)
+  {
+    GHashTable *table;
+    GHashTable *ret_table = NULL;
+    GPtrArray *foo, *bar;
+    GPtrArray *ret_foo = NULL, *ret_bar = NULL;
+
+    foo = g_ptr_array_new ();
+    g_ptr_array_add (foo, "/foo");
+
+    bar = g_ptr_array_new ();
+    g_ptr_array_add (bar, "/bar");
+    g_ptr_array_add (bar, "/baz");
+
+    table = g_hash_table_new (g_str_hash, g_str_equal);
+    g_hash_table_insert (table, "/foo", foo);
+    g_hash_table_insert (table, "/bar", bar);
+
+    g_print ("Calling DictOfObjs\n");
+
+    if (!org_freedesktop_DBus_GLib_Tests_MyObject_dict_of_objs (proxy, table,
+          &ret_table, &error))
+      lose_gerror ("Failed to complete DictOfObjs call", error);
+
+    g_ptr_array_free (foo, TRUE);
+    g_ptr_array_free (bar, TRUE);
+    g_hash_table_destroy (table);
+
+    if (ret_table == NULL)
+      lose ("DictOfObjs didn't return a hash table");
+
+    if (g_hash_table_size (ret_table) != 2)
+      lose ("DictOfObjs has too many entries");
+
+    ret_foo = g_hash_table_lookup (ret_table, "/foo");
+    ret_bar = g_hash_table_lookup (ret_table, "/bar");
+
+    if (ret_foo == NULL || ret_bar == NULL)
+      lose ("DictOfObjs is missing entries");
+
+    if (ret_foo->len != 1 ||
+        strcmp (g_ptr_array_index (ret_foo, 0), "/foo") != 0)
+      lose ("DictOfObjs mangled /foo");
+
+    if (ret_bar->len != 2 ||
+        strcmp (g_ptr_array_index (ret_bar, 0), "/bar") != 0 ||
+        strcmp (g_ptr_array_index (ret_bar, 1), "/baz") != 0)
+      lose ("DictOfObjs mangled /bar");
+
+    g_boxed_free (dbus_g_type_get_map ("GHashTable", DBUS_TYPE_G_OBJECT_PATH,
+          dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH)), ret_table);
+
+    g_mem_profile ();
+  }
+
 
 
   /* Signal handling tests */
